#!/usr/bin/env node
'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var init = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(appName) {
        var cliPath, installPath, appPath, packageManager, command;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        cliPath = __dirname; // Directory where CLI is installed.

                        installPath = path.resolve('./'); // Directory where react-native app folder is created.

                        appPath = installPath + '/' + appName; // Directory where react-native app code is created.

                        if (!fs.existsSync(appPath)) {
                            _context.next = 7;
                            break;
                        }

                        throw new Error(appName + ' directory already exists.');

                    case 7:
                        _context.next = 9;
                        return getPackageManager();

                    case 9:
                        packageManager = _context.sent;
                        _context.next = 12;
                        return createApp(appName, installPath);

                    case 12:
                        _context.next = 14;
                        return createSourceDirectory(appPath + '/' + constants.SOURCE_FOLDER_NAME);

                    case 14:
                        _context.next = 16;
                        return copyAssetFiles(cliPath + '/assets', appPath);

                    case 16:
                        _context.next = 18;
                        return modifyAppFiles(appPath, packageManager);

                    case 18:
                        _context.next = 20;
                        return installDevDeps(appPath, packageManager);

                    case 20:

                        _message2.default.success(appName + ' project created successfully.');

                        command = packageManager + (commandArgs[packageManager].run ? ' ' + commandArgs[packageManager].run : '');

                        _message2.default.info('Use \'' + command + ' android\' or \'' + command + ' ios\' to run.');

                    case 23:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    return function init(_x) {
        return _ref.apply(this, arguments);
    };
}();

/**Returns yarn if installed, otherwise use npm. */


var _chalk = require('chalk');

var chalk = _interopRequireWildcard(_chalk);

var _child_process = require('child_process');

var _path = require('path');

var path = _interopRequireWildcard(_path);

var _fs = require('fs');

var fs = _interopRequireWildcard(_fs);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _message = require('./message');

var _message2 = _interopRequireDefault(_message);

var _constants = require('./constants');

var constants = _interopRequireWildcard(_constants);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var commandArgs = {
    npm: {
        install: 'install',
        run: 'run',
        dev: function dev(deps) {
            return ['install'].concat((0, _toConsumableArray3.default)(deps), ['--save-dev']);
        }
    },
    yarn: {
        install: 'install',
        run: '',
        dev: function dev(deps) {
            return ['add'].concat((0, _toConsumableArray3.default)(deps), ['--dev']);
        }
    }
};

function main(argv) {
    var appName = getAppName(argv);

    if (!appName) {
        _message2.default.error('App name is not specified.');
        return;
    }

    init(appName).catch(_message2.default.error);
}

function getAppName(argv) {
    if (argv.length > 2) {
        return argv[2];
    }

    return '';
}

function getPackageManager() {
    return new Promise(function (resolve, reject) {
        (0, _child_process.execFile)(getCommand(constants.YARN), ['-v'], function (error, stdout, stderr) {
            if (error) {
                resolve(constants.NPM);
            } else {
                resolve(constants.YARN);
            }
        });
    });
}

function createApp(appName, installpath) {
    return new Promise(function (resolve, reject) {
        var defaults = {
            cwd: installpath,
            env: process.env
        };

        var native = (0, _child_process.spawn)(getCommand(constants.REACT_NATIVE), ['init', appName], defaults);

        native.stdout.on('data', _message2.default.info);

        native.stderr.on('data', function (data) {
            var text = data.toString();
            if (text.startsWith('Error')) {
                _message2.default.error(data);
            } else if (text.startsWith('warning')) {
                _message2.default.warn(data);
            }
        });

        native.on('error', _message2.default.error);

        native.on('close', function (code) {
            if (code !== 0) {
                reject('Child process exited with code ' + code);
                return;
            }

            resolve();
        });
    });
}

function createSourceDirectory(sourceDirectory) {
    return new Promise(function (resolve, reject) {
        fs.mkdir(sourceDirectory, function (error) {
            return error ? reject(error) : resolve();
        });
    });
}

function copyAssetFiles(cliAssetsPath, destDir) {
    var appPromise = copyAssetFile('App.tsx', cliAssetsPath, destDir + '/' + constants.SOURCE_FOLDER_NAME);
    var tsConfigPromise = copyAssetFile('tsconfig.json', cliAssetsPath, destDir);
    var tsLintPromise = copyAssetFile('tslint.json', cliAssetsPath, destDir);

    return Promise.all([appPromise, tsConfigPromise, tsLintPromise]);
}

function modifyAppFiles(appPath, packageManager) {
    var indexPromise = modifyIndexJs(appPath);
    var packagePromise = modifyPackageJson(appPath, packageManager);
    var removeAppJsPromise = removeAppJs(appPath);

    return Promise.all([indexPromise, packagePromise, removeAppJsPromise]);
}

function copyAssetFile(fileName, cliAssetsPath, destDir) {
    return copyFile(cliAssetsPath + '/' + fileName, destDir + '/' + fileName);
}

function modifyIndexJs(appPath) {
    return new Promise(function (resolve, reject) {
        var fileName = appPath + '/index.js';

        fs.readFile(fileName, 'utf8', function (error, content) {
            if (error) {
                reject(error);
                return;
            }

            var replacedContent = content.replace(/'.\/App'/i, '\'./' + constants.BUILD_FOLDER_NAME + '/App\'');

            fs.writeFile(fileName, replacedContent, 'utf8', function (error) {
                if (error) {
                    reject(error);
                    return;
                }

                resolve();
            });
        });
    });
}

function modifyPackageJson(appPath, packageManager) {
    return new Promise(function (resolve, reject) {
        var fileName = appPath + '/package.json';

        fs.readFile(fileName, 'utf8', function (error, content) {
            if (error) {
                reject(error);
                return;
            }

            var packageJson = JSON.parse(content);
            var command = packageManager + ' ' + commandArgs[packageManager].run;

            packageJson.scripts = Object.assign({}, packageJson.scripts, {
                "tsc": "tsc",
                "clean": "rimraf build",
                "build": command + ' clean && ' + command + ' tsc --',
                "lint": "tslint src/**/*.ts",
                "watch": command + ' build -w',
                "ios": command + ' build && concurrently -r "' + command + ' watch" "react-native run-ios"',
                "android": command + ' build && concurrently -r "' + command + ' watch" "react-native run-android"'
            });

            var replacedContent = JSON.stringify(packageJson, null, '\t');

            fs.writeFile(fileName, replacedContent, 'utf8', function (error) {
                if (error) {
                    reject(error);
                    return;
                }

                resolve();
            });
        });
    });
}

function removeAppJs(appPath) {
    return new Promise(function (resolve, reject) {
        fs.unlink(appPath + '/App.js', function (error) {
            return error ? reject(error) : resolve();
        });
    });
}

function installDevDeps(appPath, packageManager) {
    return new Promise(function (resolve, reject) {
        var defaults = {
            cwd: appPath,
            env: process.env
        };

        var dependencies = ['react-native-cli', 'concurrently', 'rimraf', 'typescript', 'tslint', '@types/jest', '@types/react', '@types/react-native'];

        var native = (0, _child_process.spawn)(getCommand(packageManager), commandArgs[packageManager].dev(dependencies), defaults);

        native.stdout.on('data', _message2.default.info);

        native.stderr.on('data', function (data) {
            var text = data.toString();
            if (text.startsWith('Error')) {
                _message2.default.error(data);
            } else if (text.startsWith('warning')) {
                _message2.default.warn(data);
            }
        });

        native.on('error', _message2.default.error);

        native.on('close', function (code) {
            if (code !== 0) {
                reject('Child process exited with code ' + code);
                return;
            }

            resolve();
        });
    });
}

function copyFile(source, destination) {
    return new Promise(function (resolve, reject) {
        var writeStream = fs.createWriteStream(destination).on('error', function (error) {
            return rejectCleanup;
        }).on('finish', resolve);

        var readStream = fs.createReadStream(source).on('error', function (error) {
            return rejectCleanup;
        }).pipe(writeStream);

        function rejectCleanup(error) {
            readStream.destroy();
            writeStream.end();
            reject(error);
        }
    });
}

function getCommand(command) {
    return process.platform === 'win32' ? command + '.cmd' : command;
}

main(process.argv);